package com.hello2morrow.sonargraph.languageprovider.java.command.system.scripts;

import static org.junit.Assert.*

import com.hello2morrow.sonargraph.core.model.common.Severity
import com.hello2morrow.sonargraph.core.model.programming.AggregatedDependency
import com.hello2morrow.sonargraph.core.model.script.GroovyScript
import com.hello2morrow.sonargraph.core.model.script.ScriptIssueElementTreeNode
import com.hello2morrow.sonargraph.core.model.script.ScriptNamedTreeNode
import com.hello2morrow.sonargraph.core.model.script.ScriptResultSet
import com.hello2morrow.sonargraph.core.model.script.ScriptSeverityTreeNode
import com.hello2morrow.sonargraph.languageprovider.java.foundation.common.JavaTestFixture;

public class SuperTypeUsesSubTypeScriptTest extends AbstractJavaScriptTest {
    private static final String PROJECT_PATH = JavaTestFixture.SCRIPT_SUPERTYPE_USES_SUBTYPE_PROJECT_PATH;
    private static final String SCRIPT_PATH = JavaTestFixture.SCRIPT_SUPERTYPE_USES_SUBTYPE_SCRIPT_PATH;

    public SuperTypeUsesSubTypeScriptTest() {
        super(PROJECT_PATH, SCRIPT_PATH);
    }

    @Override
    void verifyResult(GroovyScript script) {
        ScriptResultSet result = script.getResultSet();
        ScriptNamedTreeNode issues = result.getIssueRootNode();

        List<ScriptSeverityTreeNode> scriptWarnings = issues.getChildren().findAll {
            if (it instanceof ScriptSeverityTreeNode) {
                ScriptSeverityTreeNode node = (ScriptSeverityTreeNode) it;
                if (node.getSeverity() == Severity.WARNING) {
                    return node;
                }
                return null;
            }
        }
        assertEquals(1, scriptWarnings.size());

        ScriptSeverityTreeNode severityNode = scriptWarnings.get(0);
        List<ScriptIssueElementTreeNode> warningNodes = severityNode.getChildren().get(0).getChildren().findAll
        { it instanceof ScriptIssueElementTreeNode }

        assertEquals(2, warningNodes.size());

        ScriptIssueElementTreeNode node1 = warningNodes.find {
            if (!it.getDescription().equals("Reference to subtype SubType")) {
                return false;
            }
            if (! (it.getElement() instanceof AggregatedDependency)) {
                return false;
            }

            AggregatedDependency dep = (AggregatedDependency) it.getElement();
            if (!dep.getFromEndPoint().getName().equals("com.hello2morrow.sonargraph.scripttest.SuperType") || !dep.getToEndPoint().getName().equals("com.hello2morrow.sonargraph.scripttest.SubType")) {
                return false;
            }
            return true;
        }

        assertNotNull("Expected dependency 'SuperType -> SubType' not found", node1);

        ScriptIssueElementTreeNode node2 = warningNodes.find {
            if (!it.getDescription().equals("Reference to subtype SuperType")) {
                return false;
            }
            if (! (it.getElement() instanceof AggregatedDependency)) {
                return false;
            }

            AggregatedDependency dep = (AggregatedDependency) it.getElement();
            if (!dep.getFromEndPoint().getName().equals("com.hello2morrow.sonargraph.scripttest.ISuperInterface") || !dep.getToEndPoint().getName().equals("com.hello2morrow.sonargraph.scripttest.SuperType")) {
                return false;
            }
            return true;
        }
        assertNotNull("Expected dependency 'ISuperInterface -> SuperType' not found", node2);
    }
}
